package com.snowm.scanner.helpers

import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.os.Bundle
import com.google.android.gms.location.Geofence
import com.google.android.gms.location.GeofencingClient
import com.google.android.gms.location.GeofencingRequest
import com.google.android.gms.location.LocationServices
import java.util.ArrayList

class GeofencingHelper(var context: Context) {
    private var geofencingClient: GeofencingClient = LocationServices.getGeofencingClient(context)
    private var geofenceList: ArrayList<Geofence> = ArrayList()
    fun startGeoFencing(geofence: com.snowm.scanner.models.Geofence) {
        geofenceList.add(Geofence.Builder()
                .setRequestId(geofence.identifier)
                .setCircularRegion(
                        geofence.latitude, geofence.longitude, geofence.radius
                )
                .setExpirationDuration(Geofence.NEVER_EXPIRE)
                .setTransitionTypes(Geofence.GEOFENCE_TRANSITION_ENTER or Geofence.GEOFENCE_TRANSITION_EXIT)
                .build())
        geofencingClient.addGeofences(getGeofencingRequest(), geofencePendingIntent(geofence))
    }

    fun stopGepfencing(identifier: String) {
        val ids = ArrayList<String>(1)
        ids.add(identifier)
        geofencingClient.removeGeofences(ids)
    }

    fun getGeofencingRequest(): GeofencingRequest {
        return GeofencingRequest.Builder().apply {
            setInitialTrigger(GeofencingRequest.INITIAL_TRIGGER_ENTER)
            addGeofences(geofenceList)
        }.build()
    }

    private fun geofencePendingIntent(geofence: com.snowm.scanner.models.Geofence): PendingIntent {
        val intent = Intent()
        intent.setPackage(context.packageName)
        intent.action = "io.snowm.geofence"
        val bundle = Bundle()
        bundle.putSerializable("geofence", geofence.toHashMap())
        intent.putExtras(bundle)
        return PendingIntent.getBroadcast(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT)
    }
}