package com.snowm.scanner.services

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import com.google.android.gms.location.*


open class SnowMGeofenceReceiver : BroadcastReceiver() {
    lateinit var context: Context

    private var locationProviderClient: FusedLocationProviderClient? = null

    override fun onReceive(context: Context, intent: Intent) {
        this.context = context
        locationProviderClient = LocationServices.getFusedLocationProviderClient(context)
        if (intent.action == "android.intent.action.BOOT_COMPLETED") {
            onBootComplete()
        } else if (intent.action == "io.snowm.geofence") {
            val geofencingEvent = GeofencingEvent.fromIntent(intent)
            if (geofencingEvent.hasError()) {
                val errorMessage: String = GeofenceStatusCodes.getStatusCodeString(geofencingEvent.errorCode)
                println(errorMessage)
                return
            }
            geofencingEvent.triggeringGeofences.forEach {
                if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_ENTER) {
                    didEnterGeofence(it)
                    didEnterGeofence(it)
                } else if (geofencingEvent.geofenceTransition == Geofence.GEOFENCE_TRANSITION_EXIT) {
                    didExitGeofence(it)
                    didExitGeofence(it)
                }
            }
        }
    }

    open fun didEnterGeofence(geofence: Geofence) {
        println("entered")
    }

    open fun didExitGeofence(geofence: Geofence) {
        println("exited")
    }

    open fun onBootComplete() {
        println("boot complete")

    }

}