import Flutter
import UIKit
import CoreLocation

public class SwiftSnowmScannerPlugin: NSObject, FlutterPlugin {
    var beaconScanner = BeaconScanner()
    var permissionManager = PermissionManager()
    var mqttHelper = MqttHelper()
    var channel: FlutterMethodChannel?
    let backgroundScanner: BackgroundBeaconScanner = BackgroundBeaconScanner.shared
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "snowm_scanner", binaryMessenger: registrar.messenger())
        let instance = SwiftSnowmScannerPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let method = call.method
        let arguments: Dictionary<String, Any>? = call.arguments as! Dictionary<String, Any>?
        if method.contains("scanBeacons"){
            let uuids:[String]  = arguments?["uuids"] as! [String]
            //TODO: Trigger mqtt protocol on native side
            let enableMqtt:Bool  =  arguments?["enableMqtt"] as! Bool
            beaconScanner.scanBeacons(method:method, uuids: uuids) { (beacons) -> () in
                if enableMqtt {
                    beacons.forEach { (beacon) in
                        self.mqttHelper.sendBeaconData(beacons:beacon)
                    }
                }
                self.sendBeaconResponse(method:method, beacons: beacons)
            }
            result(true)
        } else if method == "requestPermission"{
            result(permissionManager.requestPermission())
        } else if method == "permissionState"{
            result(permissionManager.permissionStatus)
        } else if method == "bluetoothState"{
            result(permissionManager.state)
        }  else if method == "getCurrentScanId"{
            result(backgroundScanner.currentScannerId())
        } else if method.contains( "bluetoothStateListener"){
            permissionManager.bluetoothState(method: method) { (state) in
                self.sendBluetoothStateResponse(method: method, state: state)
            }
            result(true)
        } else if method == "cancelBluetoothStateListener"{
            let streamIdentifier = arguments?["methodName"] as! String
            permissionManager.removeBluetoothState(identifier: streamIdentifier)
            result(true)
        } else if method == "cancelStream"{
            let streamIdentifier = arguments?["methodName"] as! String
            beaconScanner.removeScanner(identifier: streamIdentifier)
            result(true)
        } else if method == "scanBeaconBackground"{
            let uuids:[String]  = arguments?["uuids"] as! [String]
            let scanId:String  = arguments?["scanId"] as! String
            let customData:Dictionary = arguments?["customData"] as! Dictionary<String, Any>
            backgroundScanner.scanBeacons(scanId:scanId, uuids: uuids, customData: customData)
            let geofences: [Dictionary<String, Any>?] = (arguments?["geofences"] as! [Dictionary<String, Any>?])
            var count = 1
            geofences.forEach { (geofence) in
                let radius: CLLocationDistance = geofence?["radius"] as! CLLocationDistance
                let point: CLLocationCoordinate2D = CLLocationCoordinate2D(latitude: geofence?["latitude"] as! CLLocationDegrees, longitude: geofence?["longitude"] as! CLLocationDegrees)
                backgroundScanner.addGeoFencing(identifier: "backgroundGeofence#\(count)",location: point, radius: radius)
                count+=1
            }
            result(true)
        } else if method == "stopBackgroundScan"{
            backgroundScanner.stopBackgroundScanning()
            result(true)
        }
    }
    
    private func sendBluetoothStateResponse(method:String, state:String){
        channel?.invokeMethod(method, arguments: state)
    }
    
    private func sendBeaconResponse(method:String, beacons: [SnowMBeacon]){
        var response: [AnyHashable:Any] = [AnyHashable:Any]();
        response["beacons"] = beacons.map{ $0.toObject()};
        channel?.invokeMethod(method, arguments: response)
    }
}
